// Copyright © 2021 Finn Herzfeld <finn@janky.solutions>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package account

import (
	"log"

	"github.com/spf13/cobra"

	"gitlab.com/signald/signald-go/signald/client-protocol/v0"
)

var (
	actionLink        bool
	actionRequestCode bool
	actionVerify      bool

	phoneNumber string
	code        string
	voice       bool
	noWait      bool

	CreateAccountCmd = &cobra.Command{
		Use:     "account",
		Aliases: []string{"accounts"},
		Short:   "create an account",
		PreRun: func(_ *cobra.Command, _ []string) {

			actionCount := 0
			if actionLink {
				actionCount++
			}
			if actionRequestCode {
				actionCount++
			}
			if actionVerify {
				actionCount++
			}
			if actionCount > 1 {
				log.Fatal("invalid arguments: must select exactly one of --request-code, --verify or --link")
			}

			if actionLink {
				if phoneNumber != "" {
					log.Fatal("cannot use --phone-number with --link")
				}
				if code != "" {
					log.Fatal("cannot use --code with --link")
				}
				if voice {
					log.Fatal("cannot use --voice with --link")
				}
			}

			if actionRequestCode {
				if phoneNumber == "" {
					log.Fatal("--phone-number required")
				}
				if code != "" {
					log.Fatal("cannot use --code with --request-code")
				}
				if noWait {
					log.Fatal("cannot use --no-wait with --request-code")
				}
			}

			if actionVerify {
				if phoneNumber == "" {
					log.Fatal("--phone-number required")
				}
				if code == "" {
					log.Fatal("--code required")
				}
				if voice {
					log.Fatal("cannot use --voice with --verify")
				}
				if noWait {
					log.Fatal("cannot use --no-wait with --verify")
				}
			}
		},
		Run: func(_ *cobra.Command, _ []string) {
			if actionLink {
				link()
			}

			if actionRequestCode {
				requestCode()
			}

			if actionVerify {
				verify()
			}
		},
	}
)

func init() {
	CreateAccountCmd.Flags().BoolVarP(&actionLink, "link", "l", false, "link an existing account")
	CreateAccountCmd.Flags().BoolVarP(&actionRequestCode, "request-code", "r", false, "request a verification code")
	CreateAccountCmd.Flags().BoolVarP(&actionVerify, "verify", "v", false, "submit a verification code and complete a new account registration")

	CreateAccountCmd.Flags().StringVarP(&phoneNumber, "number", "n", "", "only with --request-code or --verify. phone number being registered")
	CreateAccountCmd.Flags().StringVarP(&code, "code", "c", "", "only with --verify. verification code to submit")
	CreateAccountCmd.Flags().BoolVarP(&voice, "voice", "V", false, "only with --request-code. request verification code be sent via an automated voice call (code is sent via SMS by default)")
	CreateAccountCmd.Flags().BoolVar(&noWait, "no-wait", false, "only with --link. return after the linking URI is printed to stdout. By default, --link will wait until the server acknowledges that the linking has completed")
}

func getResponse(c chan v0.LegacyResponse, id string) v0.LegacyResponse {
	for {
		message := <-c
		if message.ID == id {
			return message
		}
	}
}
