// Copyright © 2021 Finn Herzfeld <finn@janky.solutions>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package group

import (
	"encoding/json"
	"log"
	"os"

	"github.com/jedib0t/go-pretty/v6/table"
	"github.com/spf13/cobra"
	"gopkg.in/yaml.v2"

	"gitlab.com/signald/signald-go/cmd/signaldctl/common"
	"gitlab.com/signald/signald-go/signald/client-protocol/v1"
)

var (
	account     string
	groupID     string
	GetGroupCmd = &cobra.Command{
		Use:     "group",
		Aliases: []string{"groups"},
		Short:   "return a list of Signal groups",
		Run: func(_ *cobra.Command, _ []string) {
			go common.Signald.Listen(nil)
			var resp v1.GroupList
			if len(groupID) == 0 || len(groupID) == 24 {
				var err error
				req := v1.ListGroupsRequest{Account: account}
				resp, err = req.Submit(common.Signald)
				if err != nil {
					log.Fatal(err, "error communicating with signald")
				}

				if len(groupID) == 24 {
					for _, group := range resp.LegacyGroups {
						if group.GroupId == groupID {
							resp = v1.GroupList{LegacyGroups: []*v1.JsonGroupInfo{group}}
							break
						}
					}
					log.Fatal("group not found")
				}
			} else if len(groupID) == 44 {
				req := v1.GetGroupRequest{Account: account, GroupID: groupID}
				group, err := req.Submit(common.Signald)
				if err != nil {
					log.Fatal(err, "error communicating with signald")
				}
				resp = v1.GroupList{Groups: []*v1.JsonGroupV2Info{&group}}
			} else {
				log.Fatal("malformed group ID (expected length of 24 or 44)")
			}

			switch common.OutputFormat {
			case common.OutputFormatJSON:
				err := json.NewEncoder(os.Stdout).Encode(resp)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatYAML:
				err := yaml.NewEncoder(os.Stdout).Encode(resp)
				if err != nil {
					log.Fatal(err, "error encoding response to stdout")
				}
			case common.OutputFormatCSV, common.OutputFormatTable, common.OutputFormatDefault:
				t := table.NewWriter()
				t.SetOutputMirror(os.Stdout)
				t.AppendHeader(table.Row{"ID", "Title", "Members"})

				for _, group := range resp.Groups {
					t.AppendRow(table.Row{group.ID, group.Title, len(group.Members)})
				}

				for _, group := range resp.LegacyGroups {
					t.AppendRow(table.Row{group.GroupId, group.Name, len(group.Members)})
				}

				if common.OutputFormat == common.OutputFormatCSV {
					t.RenderCSV()
				} else {
					t.Render()
				}
			default:
				log.Fatal("Unsupported output format")
			}
		},
	}
)

func init() {
	GetGroupCmd.Flags().StringVarP(&account, "account", "a", "", "the signald account to use")
	GetGroupCmd.Flags().StringVarP(&groupID, "group-id", "g", "", "if set, fetches the latest state of a group from the server (local state is used for legacy groups). If unset, return all groups")
	common.Must(GetGroupCmd.MarkFlagRequired("account"))
}
